@@ -37,6 +37,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Properties;
+import java.util.Random;
 import java.util.TreeMap;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -81,6 +82,7 @@ import org.apache.pig.backend.hadoop.executionengine.mapReduceLayer.partitioners
 import org.apache.pig.backend.hadoop.executionengine.mapReduceLayer.plans.MROperPlan;
 import org.apache.pig.backend.hadoop.executionengine.physicalLayer.PhysicalOperator;
 import org.apache.pig.backend.hadoop.executionengine.physicalLayer.expressionOperators.ConstantExpression;
+import org.apache.pig.backend.hadoop.executionengine.physicalLayer.expressionOperators.POUserFunc;
 import org.apache.pig.backend.hadoop.executionengine.physicalLayer.plans.PhyPlanVisitor;
 import org.apache.pig.backend.hadoop.executionengine.physicalLayer.plans.PhysicalPlan;
 import org.apache.pig.backend.hadoop.executionengine.physicalLayer.plans.UdfCacheShipFilesVisitor;
@@ -172,6 +174,9 @@ public class JobControlCompiler{
     public static final String PIG_MAP_SEPARATOR = "_";
     public HashMap<String, ArrayList<Pair<String,Long>>> globalCounters = new HashMap<String, ArrayList<Pair<String,Long>>>();
 
+    private static final Random RAND = new Random();
+    private static final String CACHE_TMP_FILE_TEMPLATE = "tmp%d.tmp";
+
     public static final String SMALL_JOB_LOG_MSG = "This job was detected as a small job, will run in-process instead";
     public static final String BIG_JOB_LOG_MSG = "This job cannot be converted run in-process";
 
@@ -508,7 +513,7 @@ public class JobControlCompiler{
 
         Configuration conf = nwJob.getConfiguration();
 
-        ArrayList<FileSpec> inp = new ArrayList<FileSpec>();
+        ArrayList<POLoad> inp = new ArrayList<POLoad>();
         ArrayList<List<OperatorKey>> inpTargets = new ArrayList<List<OperatorKey>>();
         ArrayList<String> inpSignatureLists = new ArrayList<String>();
         ArrayList<Long> inpLimits = new ArrayList<Long>();
@@ -547,12 +552,20 @@ public class JobControlCompiler{
                 for (POLoad ld : lds) {
                     LoadFunc lf = ld.getLoadFunc();
                     lf.setLocation(ld.getLFile().getFileName(), nwJob);
-
+                    lf.addCredentials(nwJob.getCredentials(), conf);
+                    ld.setParentPlan(null);
                     //Store the inp filespecs
-                    inp.add(ld.getLFile());
+                    inp.add(ld);
                 }
             }
 
+            //Process the POUserFunc
+            List<POUserFunc> userFuncs = PlanHelper.getPhysicalOperators(mro.mapPlan, POUserFunc.class);
+            userFuncs.addAll(PlanHelper.getPhysicalOperators(mro.reducePlan, POUserFunc.class));
+            for (POUserFunc userFunc : userFuncs) {
+                userFunc.getFunc().addCredentials(nwJob.getCredentials(), conf);
+            }
+
             if(!mro.reducePlan.isEmpty()){
                 log.info("Reduce phase detected, estimating # of required reducers.");
                 adjustNumReducers(plan, mro, nwJob);
@@ -704,7 +717,7 @@ public class JobControlCompiler{
             if(Utils.isLocal(pigContext, conf)) {
                 ConfigurationUtil.replaceConfigForLocalMode(conf);
             }
-            conf.set(PigInputFormat.PIG_INPUTS, ObjectSerializer.serialize(inp));
+            conf.set(PigInputFormat.PIG_LOADS, ObjectSerializer.serialize(inp));
             conf.set(PigInputFormat.PIG_INPUT_TARGETS, ObjectSerializer.serialize(inpTargets));
             conf.set(PigInputFormat.PIG_INPUT_SIGNATURES, ObjectSerializer.serialize(inpSignatureLists));
             conf.set(PigInputFormat.PIG_INPUT_LIMITS, ObjectSerializer.serialize(inpLimits));
@@ -769,6 +782,7 @@ public class JobControlCompiler{
                         osf.cleanupOutput(st, nwJob);
                     }
                 }
+                sFunc.addCredentials(nwJob.getCredentials(), conf);
             }
 
             for (POStore st : reduceStores) {
@@ -781,6 +795,7 @@ public class JobControlCompiler{
                         osf.cleanupOutput(st, nwJob);
                     }
                 }
+                sFunc.addCredentials(nwJob.getCredentials(), conf);
             }
 
             setOutputFormat(nwJob);
@@ -1733,7 +1748,6 @@ public class JobControlCompiler{
             URL url) throws IOException {
         InputStream is1 = null;
         InputStream is2 = null;
-        OutputStream os = null;
 
         try {
             Path stagingDir = getCacheStagingDir(conf);
@@ -1756,10 +1770,22 @@ public class JobControlCompiler{
             is2 = url.openStream();
             short replication = (short)conf.getInt(PigConfiguration.PIG_USER_CACHE_REPLICATION,
                     conf.getInt("mapred.submit.replication", 10));
-            os = fs.create(cacheFile, replication);
-            fs.setPermission(cacheFile, FileLocalizer.OWNER_ONLY_PERMS);
-            IOUtils.copyBytes(is2, os, 4096, true);
+            Path tempCacheFile = new Path(cacheDir,
+                String.format(CACHE_TMP_FILE_TEMPLATE, RAND.nextInt()));
 
+            try {
+                try (OutputStream os = fs.create(tempCacheFile, replication)) {
+                    fs.setPermission(tempCacheFile, FileLocalizer.OWNER_ONLY_PERMS);
+                    IOUtils.copyBytes(is2, os, 4096, true);
+                }
+                fs.rename(tempCacheFile, cacheFile);
+            } catch (IOException ioe) {
+                // Attempt some cleanup to avoid leaving tmp files around
+                if (fs.exists(tempCacheFile)) {
+                    fs.delete(tempCacheFile, false);
+                }
+                throw ioe;
+            }
             return cacheFile;
 
         } catch (IOException ioe) {
@@ -1768,10 +1794,6 @@ public class JobControlCompiler{
         } finally {
             org.apache.commons.io.IOUtils.closeQuietly(is1);
             org.apache.commons.io.IOUtils.closeQuietly(is2);
-            // IOUtils should not close stream to HDFS quietly
-            if (os != null) {
-                os.close();
-            }
         }
     }
 
