@@ -360,11 +360,30 @@ public abstract class PhysicalOperator extends Operator<PhyPlanVisitor> implemen
             default:
                 throw new ExecException("Unsupported type for getNext: " + DataType.findTypeName(dataType));
             }
+        } catch (ExecException e) {
+            throw new ExecException("Exception while executing for " + originalLocationsToDescriptiveString()
+                                    + ": " + e.toString(), e.getErrorCode(), e);
         } catch (RuntimeException e) {
             throw new ExecException("Exception while executing " + this.toString() + ": " + e.toString(), e);
         }
     }
 
+    private String originalLocationsToDescriptiveString() {
+        StringBuilder sb = new StringBuilder();
+        sb.append('[');
+        boolean isFirstItem = true;
+        for( OriginalLocation ol : getOriginalLocations() ) {
+            if( isFirstItem ) {
+                isFirstItem = false;
+            } else {
+                sb.append(",");
+            }
+            sb.append(ol.getAlias()).append("[line=").append(ol.getLine())
+              .append(",offset=").append(ol.getOffset()).append("]");
+        }
+        return sb.append(']').toString();
+    }
+
     public Result getNextInteger() throws ExecException {
         return res;
     }
@@ -441,6 +460,10 @@ public abstract class PhysicalOperator extends Operator<PhyPlanVisitor> implemen
     public void reset() {
     }
 
+    public boolean isEndOfAllInput() {
+        return parentPlan.endOfAllInput;
+    }
+
     /**
      * @return PigProgressable stored in threadlocal
      */
@@ -493,6 +516,10 @@ public abstract class PhysicalOperator extends Operator<PhyPlanVisitor> implemen
        parentPlan = physicalPlan;
     }
 
+    public PhysicalPlan getParentPlan() {
+        return parentPlan;
+    }
+
     public Log getLogger() {
         return log;
     }
