@@ -19,6 +19,7 @@ package org.apache.pig.data;
 
 import java.util.Comparator;
 import java.util.List;
+import java.util.Set;
 
 /**
  * Default implementation of BagFactory.
@@ -76,6 +77,21 @@ public class DefaultBagFactory extends BagFactory {
         return b;
     }
 
+    /**
+     * Get a distinct data bag.
+     * @param tuples Distinct set of tuples used to initialize the bag.
+     * If null, an empty bag is returned.
+     */
+    @Override
+    public DataBag newDistinctBag(Set<Tuple> tuples) {
+        if (tuples == null) {
+            return newDistinctBag();
+        }
+
+        DataBag b = new DistinctDataBag(tuples);
+        return b;
+    }
+
     DefaultBagFactory() {
         super();
     }
