@@ -17,6 +17,9 @@
  */
 package org.apache.pig.impl;
 
+import com.google.common.base.Splitter;
+import com.google.common.collect.Lists;
+
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
 import java.io.File;
@@ -26,6 +29,8 @@ import java.io.FileWriter;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.io.StringWriter;
 import java.lang.reflect.Constructor;
@@ -897,6 +902,11 @@ public class PigContext implements Serializable {
     {
         defaultLogLevel = l;
     }
+ 
+   public int getDefaultParallel() {
+        return defaultParallel;
+    }
+
     public static ClassLoader getClassLoader() {
         return classloader;
     }
@@ -906,5 +916,5 @@ public class PigContext implements Serializable {
         } else {
             classloader = new ContextClassLoader(cl);
         }
-    }
+   }
 }
