@@ -334,7 +334,12 @@ public abstract class FilterExtractor {
 
         for( Operator succ : children ) {
             filteredPlan.disconnect( op, succ );
-            removeFromFilteredPlan( succ );
+            // check if this successor has any other predecessor.
+            // if none, proceed with the removal.
+            List<Operator> predsOfsucc = filteredPlan.getPredecessors( succ );
+            if( predsOfsucc == null || predsOfsucc.size() == 0 ) {
+                removeFromFilteredPlan( succ );
+            }
         }
 
         filteredPlan.remove( op );
@@ -351,7 +356,7 @@ public abstract class FilterExtractor {
         } else if (op instanceof ProjectExpression) {
             ProjectExpression projExpr = (ProjectExpression)op;
             String fieldName = projExpr.getFieldSchema().alias;
-            return new Expression.Column(fieldName);
+            return new Expression.Column(fieldName, projExpr.getType());
         } else if(op instanceof BinaryExpression) {
             BinaryExpression binOp = (BinaryExpression)op;
             if(binOp instanceof AddExpression) {
