@@ -33,6 +33,7 @@ import org.apache.pig.newplan.logical.relational.LogicalSchema.LogicalFieldSchem
 public class LOGenerate extends LogicalRelationalOperator {
      private List<LogicalExpressionPlan> outputPlans;
      private boolean[] flattenFlags;
+     private int[] flattenNumFields;
      // mUserDefinedSchema is the original input from the user, we don't suppose
      // to store uid in mUserDefinedSchema
      private List<LogicalSchema> mUserDefinedSchema = null;
@@ -111,8 +112,9 @@ public class LOGenerate extends LogicalRelationalOperator {
                             if (fieldSchema.type == DataType.BAG) {
                                 // if it is bag, get the schema of tuples
                                 if (fieldSchema.schema!=null) {
-                                    if (fieldSchema.schema.getField(0).schema!=null)
+                                    if (fieldSchema.schema.getField(0).schema!=null) {
                                         innerFieldSchemas = fieldSchema.schema.getField(0).schema.getFields();
+                                    }
                                     for (LogicalSchema.LogicalFieldSchema fs : innerFieldSchemas) {
                                         fs.alias = fs.alias == null ? null : fieldSchema.alias + "::" + fs.alias;
                                     }
@@ -198,10 +200,19 @@ public class LOGenerate extends LogicalRelationalOperator {
             }
             outputPlanSchemas.add(planSchema);
         }
+
+        flattenNumFields = new int[outputPlans.size()];
         if (schema==null || schema.size()==0) {
             schema = null;
             outputPlanSchemas = null;
+        } else {
+            for(int i=0; i<outputPlans.size(); i++) {
+                if (flattenFlags[i]) {
+                    flattenNumFields[i] = outputPlanSchemas.get(i).size();
+                }
+            }
         }
+
         
         return schema;
     }
@@ -213,6 +224,10 @@ public class LOGenerate extends LogicalRelationalOperator {
     public boolean[] getFlattenFlags() {
         return flattenFlags;
     }
+
+    public int [] getFlattenNumFields() {
+        return flattenNumFields;
+    }
     
     public void setFlattenFlags(boolean[] flatten) {
         flattenFlags = flatten;
