@@ -17,26 +17,19 @@
  */
 package org.apache.pig.builtin;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
-
 import java.io.File;
-import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.math.BigDecimal;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import java.util.TimeZone;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.fs.PathFilter;
 import org.apache.hadoop.hive.ql.io.orc.CompressionKind;
 import org.apache.hadoop.hive.ql.io.orc.OrcFile;
 import org.apache.hadoop.hive.ql.io.orc.OrcStruct;
@@ -46,7 +39,6 @@ import org.apache.hadoop.hive.serde2.io.ByteWritable;
 import org.apache.hadoop.hive.serde2.io.DoubleWritable;
 import org.apache.hadoop.hive.serde2.io.HiveDecimalWritable;
 import org.apache.hadoop.hive.serde2.io.ShortWritable;
-import org.apache.hadoop.hive.serde2.io.TimestampWritable;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspector;
 import org.apache.hadoop.hive.serde2.objectinspector.StructObjectInspector;
 import org.apache.hadoop.io.BooleanWritable;
@@ -57,13 +49,16 @@ import org.apache.hadoop.io.LongWritable;
 import org.apache.hadoop.io.Text;
 import org.apache.pig.PigServer;
 import org.apache.pig.backend.hadoop.datastorage.ConfigurationUtil;
+import org.apache.pig.builtin.mock.Storage.Data;
 import org.apache.pig.data.BinSedesTuple;
 import org.apache.pig.data.DataByteArray;
 import org.apache.pig.data.DataType;
 import org.apache.pig.data.DefaultDataBag;
 import org.apache.pig.data.Tuple;
+import org.apache.pig.hive.HiveShims;
 import org.apache.pig.impl.logicalLayer.schema.Schema;
 import org.apache.pig.test.Util;
+
 import org.joda.time.DateTime;
 import org.junit.After;
 import org.junit.Assert;
@@ -71,6 +66,13 @@ import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
+import static org.apache.pig.builtin.mock.Storage.resetData;
+import static org.apache.pig.builtin.mock.Storage.tuple;
+import static org.apache.pig.builtin.mock.Storage.bag;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertTrue;
+
 public class TestOrcStorage {
     final protected static Log LOG = LogFactory.getLog(TestOrcStorage.class);
 
@@ -86,6 +88,11 @@ public class TestOrcStorage {
     private static PigServer pigServer = null;
     private static FileSystem fs;
 
+    static {
+        System.setProperty("user.timezone", "UTC");
+        TimeZone.setDefault(TimeZone.getTimeZone("UTC"));
+    }
+
     @BeforeClass
     public static void oneTimeSetup(){
         if(Util.WINDOWS){
@@ -125,7 +132,7 @@ public class TestOrcStorage {
 
     @Test
     public void testSimpleLoad() throws Exception {
-        pigServer.registerQuery("A = load '" + basedir + "orc-file-11-format.orc'" + " using OrcStorage();" );
+        pigServer.registerQuery("A = load '" + basedir + "*-file-11-format.orc'" + " using OrcStorage();");
         Schema s = pigServer.dumpSchema("A");
         assertEquals(s.toString(), "{boolean1: boolean,byte1: int,short1: int,int1: int,long1: long," +
                 "float1: float,double1: double,bytes1: bytearray,string1: chararray," +
@@ -176,6 +183,20 @@ public class TestOrcStorage {
         }
         assertEquals(count, 10000);
     }
+    
+    @Test
+    public void testDate() throws Exception {
+        pigServer.registerQuery("A = load '" + basedir + "datetype.orc'" + " using OrcStorage();" );
+        Schema schema = pigServer.dumpSchema("A");
+        assertEquals(schema.size(), 1);
+        assertEquals(schema.getField(0).type, DataType.DATETIME);
+        Iterator<Tuple> iter = pigServer.openIterator("A");
+        Tuple t = iter.next();
+        assertTrue(t.get(0) instanceof DateTime);
+        assertEquals(t.toString(), "(2022-07-07T00:00:00.000Z)");
+        assertEquals(iter.next().toString(), "(2022-09-09T00:00:00.000Z)");
+        assertFalse(iter.hasNext());
+    }
 
     @Test
     // See PIG-4218
@@ -195,7 +216,7 @@ public class TestOrcStorage {
         Reader reader = OrcFile.createReader(fs, Util.getFirstPartFile(new Path(OUTPUT1)));
         assertEquals(reader.getNumberOfRows(), 2);
 
-        RecordReader rows = reader.rows(null);
+        RecordReader rows = reader.rows();
         Object row = rows.next(null);
         StructObjectInspector soi = (StructObjectInspector)reader.getObjectInspector();
         IntWritable intWritable = (IntWritable)soi.getStructFieldData(row,
@@ -279,10 +300,86 @@ public class TestOrcStorage {
         Tuple t = iter.next();
         assertTrue(t.toString().startsWith("(false,1,1024,65536,9223372036854775807,1.0,-15.0," +
                 ",hi,({(1,bye),(2,sigh)}),{(3,good),(4,bad)},[],"));
-        assertTrue(t.get(12).toString().matches("2000-03-12T15:00:00.000.*"));
+        assertTrue(t.get(12).toString().matches("2000-03-12T15:00:00\\.000Z.*"));
         assertTrue(t.toString().endsWith(",12345678.6547456)"));
     }
 
+    @Test
+    // See PIG-5383
+    public void testByteArrayStore() throws Exception {
+        Data data = resetData(pigServer);
+        data.set("foo", "intButTypeByteArray:bytearray",
+            tuple(1),
+            tuple(2),
+            tuple(3)
+        );
+
+        // Emunlating the case when "bytearray" represents an unknown type
+
+        pigServer.registerQuery("A = LOAD 'foo' USING mock.Storage();");
+        pigServer.registerQuery("store A into '" + OUTPUT1 + "' using OrcStorage();");
+        pigServer.registerQuery("B = load '" + OUTPUT1 + "' using OrcStorage();");
+        Iterator <Tuple> iter = pigServer.openIterator("B");
+        assertEquals(Integer.valueOf(1), DataType.toInteger(iter.next().get(0), DataType.BYTEARRAY));
+        assertEquals(Integer.valueOf(2), DataType.toInteger(iter.next().get(0), DataType.BYTEARRAY));
+        assertEquals(Integer.valueOf(3), DataType.toInteger(iter.next().get(0), DataType.BYTEARRAY));
+        assertFalse(iter.hasNext());
+    }
+
+    @Test
+    // See PIG-____
+    public void testSingleItemTuple() throws Exception {
+        Data data = resetData(pigServer);
+        data.set("foo", "a:bag{t:tuple(number:int)}",
+            tuple(bag(tuple(1), tuple(2), tuple(3))),
+            tuple(bag(tuple(4), tuple(5), tuple(6))),
+            tuple(bag(tuple(7), tuple(8), tuple(9)))
+        );
+
+        // Testing writes
+
+        //A: (a:bag{t:tuple(number:int)})
+        pigServer.registerQuery("A = LOAD 'foo' USING mock.Storage();");
+        pigServer.registerQuery("store A into '" + OUTPUT1 + "' using OrcStorage();");
+        pigServer.registerQuery("store A into '" + OUTPUT2 + "' using OrcStorage('-k');");
+
+        Reader reader1 = OrcFile.createReader(fs, Util.getFirstPartFile(new Path(OUTPUT1)));
+        //Note, "number" alias is dropped but this is an issue on Hive/OrcFile side
+        assertEquals("struct<a:array<int>>", reader1.getObjectInspector().getTypeName());
+
+        Reader reader2 = OrcFile.createReader(fs, Util.getFirstPartFile(new Path(OUTPUT2)));
+        assertEquals("struct<a:array<struct<number:int>>>", reader2.getObjectInspector().getTypeName());
+
+        assertEquals(reader1.getNumberOfRows(), 3);
+        assertEquals(reader2.getNumberOfRows(), 3);
+
+        // For read, option '-k' is ignored.  It all maps back to single tuple
+        // since Pig doesn't support Bag with primitive types.
+        pigServer.registerQuery("B = load '" + OUTPUT1 + "' using OrcStorage();");
+        pigServer.registerQuery("C = load '" + OUTPUT2 + "' using OrcStorage();");
+
+        Schema schema1 = pigServer.dumpSchema("B"); // struct<a:array<int>> --> a:{(int)}
+        Schema schema2 = pigServer.dumpSchema("C"); // struct<a:array<struct<number:int>>> --> a:{(number:int)}
+
+        // Currently Hive's OrcFile doesn't seem to store the name of the fields
+        // except for Tuples. Thus, only checking the types but not aliases.
+        // equals(schema, other, relaxInner=false, relaxAlias=true)
+        assertTrue(Schema.equals(schema1, schema2, false, true));
+
+        System.err.println(schema1);
+        System.err.println(schema2);
+        Iterator<Tuple> iter1 = pigServer.openIterator("B");
+        Iterator<Tuple> iter2 = pigServer.openIterator("C");
+        int count=0;
+        while (iter1.hasNext()) {
+          Tuple t1 = iter1.next();
+          Tuple t2 = iter2.next();
+          assertEquals(t1, t2);
+          count++;
+        }
+        assertEquals(3, count);
+    }
+
     private void verifyData(Path orcFile, Iterator<Tuple> iter, FileSystem fs, int expectedTotalRows) throws Exception {
 
         int expectedRows = 0;
@@ -291,7 +388,7 @@ public class TestOrcStorage {
         ObjectInspector oi = orcReader.getObjectInspector();
         StructObjectInspector soi = (StructObjectInspector) oi;
 
-        RecordReader reader = orcReader.rows(null);
+        RecordReader reader = orcReader.rows();
         Object row = null;
 
         while (reader.hasNext()) {
@@ -326,9 +423,9 @@ public class TestOrcStorage {
         Reader orcReaderActual = OrcFile.createReader(fs, orcFile);
         StructObjectInspector soiActual = (StructObjectInspector) orcReaderActual.getObjectInspector();
 
-        RecordReader readerExpected = orcReaderExpected.rows(null);
+        RecordReader readerExpected = orcReaderExpected.rows();
         Object expectedRow = null;
-        RecordReader readerActual = orcReaderActual.rows(null);
+        RecordReader readerActual = orcReaderActual.rows();
         Object actualRow = null;
 
         while (readerExpected.hasNext()) {
@@ -381,9 +478,9 @@ public class TestOrcStorage {
         } else if (expected instanceof BooleanWritable) {
             assertEquals(Boolean.class, actual.getClass());
             assertEquals(((BooleanWritable) expected).get(), actual);
-        } else if (expected instanceof TimestampWritable) {
+        } else if (HiveShims.TimestampWritableShim.isAssignableFrom(expected)) {
             assertEquals(DateTime.class, actual.getClass());
-            assertEquals(((TimestampWritable) expected).getTimestamp().getTime(),
+            assertEquals(HiveShims.TimestampWritableShim.millisFromTimestampWritable(expected),
                     ((DateTime) actual).getMillis());
         } else if (expected instanceof BytesWritable) {
             assertEquals(DataByteArray.class, actual.getClass());
