@@ -72,7 +72,7 @@ import org.junit.BeforeClass;
 import org.junit.Test;
 
 public class TestGrunt {
-    static MiniGenericCluster cluster = MiniGenericCluster.buildCluster();
+    static MiniGenericCluster cluster = MiniGenericCluster.buildCluster(2, 4);
     private String basedir = "test/org/apache/pig/test/data";
 
     @BeforeClass
@@ -915,16 +915,7 @@ public class TestGrunt {
     }
 
     @Test
-    public void testKeepGoigFailed() throws Throwable {
-        // in mr mode, the output file 'baz' will be automatically deleted if the mr job fails
-        // when "cat baz;" is executed, it throws "Encountered IOException. Directory baz does not exist"
-        // in GruntParser#processCat() and variable "caught" is true
-        // in spark mode, the output file 'baz' will not be automatically deleted even the job fails(see SPARK-7953)
-        // when "cat baz;" is executed, it does not throw exception and the variable "caught" is false
-        // TODO: Enable this for Spark when SPARK-7953 is resolved
-        Assume.assumeTrue(
-            "Skip this test for Spark until SPARK-7953 is resolved!",
-            !Util.isSparkExecType(cluster.getExecType()));
+    public void testKeepGoingFailed() throws Throwable {
         PigServer server = new PigServer(cluster.getExecType(), cluster.getProperties());
         PigContext context = server.getPigContext();
         Util.copyFromLocalToCluster(cluster, "test/org/apache/pig/test/data/passwd", "passwd");
