@@ -36,7 +36,7 @@ import org.apache.hadoop.mapreduce.lib.input.FileSplit;
 import org.apache.log4j.FileAppender;
 import org.apache.log4j.Level;
 import org.apache.log4j.Logger;
-import org.apache.log4j.SimpleLayout;
+import org.apache.log4j.PatternLayout;
 import org.apache.pig.PigConfiguration;
 import org.apache.pig.PigRunner;
 import org.apache.pig.PigServer;
@@ -71,7 +71,7 @@ public class TestTmpFileCompression {
         Logger logger = Logger.getLogger(clazz);
         logger.removeAllAppenders();
         logger.setLevel(Level.DEBUG);
-        SimpleLayout layout = new SimpleLayout();
+        PatternLayout layout = new PatternLayout();
         logFile = File.createTempFile("log", "");
         FileAppender appender = new FileAppender(layout, logFile.toString(),
                         false, false, 0);
