@@ -59,7 +59,6 @@ import org.apache.log4j.FileAppender;
 import org.apache.log4j.Level;
 import org.apache.log4j.Logger;
 import org.apache.log4j.PatternLayout;
-import org.apache.log4j.SimpleLayout;
 import org.apache.log4j.WriterAppender;
 import org.apache.pig.ExecType;
 import org.apache.pig.ExecTypeProvider;
@@ -105,6 +104,7 @@ import org.apache.pig.parser.ParserException;
 import org.apache.pig.parser.QueryParserDriver;
 import org.apache.pig.tools.grunt.GruntParser;
 import org.apache.pig.tools.pigstats.ScriptState;
+import org.apache.spark.package$;
 import org.junit.Assert;
 
 import com.google.common.base.Function;
@@ -400,7 +400,7 @@ public class Util {
         }
         FileStatus fileStatus = fs.getFileStatus(path);
         FileStatus[] files;
-        if (fileStatus.isDir()) {
+        if (fileStatus.isDirectory()) {
             files = fs.listStatus(path, new PathFilter() {
                 @Override
                 public boolean accept(Path p) {
@@ -535,16 +535,10 @@ public class Util {
      * @param actualResults Result of the executed Pig query
      * @param expectedResults Expected results List to validate against
      */
-     static public void checkQueryOutputs(Iterator<Tuple> actualResults,
+      static public void checkQueryOutputs(Iterator<Tuple> actualResults,
                                      List<Tuple> expectedResults) {
-         int count = 0;
-         for (Tuple expected : expectedResults) {
-             Tuple actual = actualResults.next();
-             count++;
-             Assert.assertEquals(expected.toString(), actual.toString());
-         }
-         Assert.assertEquals(expectedResults.size(), count);
-     }
+          checkQueryOutputs(actualResults, expectedResults.iterator(), null );
+      }
 
      /**
       * Helper function to check if the result of a Pig Query is in line with
@@ -560,7 +554,15 @@ public class Util {
               Tuple expected = expectedResults.next();
               Assert.assertTrue("Actual result has less records than expected results", actualResults.hasNext());
               Tuple actual = actualResults.next();
-              Assert.assertEquals(expected.toString(), actual.toString());
+
+              // If this tuple contains any bags, bags will be sorted before comparisons
+              if( !expected.equals(actual) ) {
+                  // Using string comparisons since error message is more readable
+                  // (only showing the part which differs)
+                  Assert.assertEquals(expected.toString(), actual.toString());
+                  // if above goes through, simply failing with object comparisons
+                  Assert.assertEquals(expected, actual);
+              }
               count++;
           }
           Assert.assertFalse("Actual result has more records than expected results", actualResults.hasNext());
@@ -613,9 +615,7 @@ public class Util {
          Collections.sort(actualResList);
          Collections.sort(expectedResList);
 
-         Assert.assertEquals("Comparing actual and expected results. ",
-                 expectedResList, actualResList);
-
+         checkQueryOutputs(actualResList.iterator(), expectedResList);
     }
 
     /**
@@ -731,7 +731,7 @@ public class Util {
 
         String line = null;
  	   FileStatus fst = fs.getFileStatus(new Path(fileNameOnCluster));
- 	   if(fst.isDir()) {
+ 	   if(fst.isDirectory()) {
  	       throw new IOException("Only files from cluster can be copied locally," +
  	       		" " + fileNameOnCluster + " is a directory");
  	   }
@@ -1110,7 +1110,7 @@ public class Util {
         Logger logger = Logger.getLogger(clazz);
         logger.removeAllAppenders();
         logger.setLevel(Level.DEBUG);
-        SimpleLayout layout = new SimpleLayout();
+        PatternLayout layout = new PatternLayout();
         File newLogFile = File.createTempFile("log", "");
         FileAppender appender = new FileAppender(layout, newLogFile.toString(),
                         false, false, 0);
@@ -1250,13 +1250,14 @@ public class Util {
         LogicalSchema resultSchema = org.apache.pig.impl.util.Utils.parseSchema(schemaString);
         checkQueryOutputsAfterSortRecursive(actualResultsIt, expectedResArray, resultSchema);
     }
-          /**
+    
+    /**
      * Helper function to check if the result of a Pig Query is in line with
      * expected results. It sorts actual and expected string results before comparison
      *
      * @param actualResultsIt Result of the executed Pig query
      * @param expectedResArray Expected string results to validate against
-     * @param fs fieldSchema of expecteResArray
+     * @param schema fieldSchema of expecteResArray
      * @throws IOException
      */
     static public void checkQueryOutputsAfterSortRecursive(Iterator<Tuple> actualResultsIt,
@@ -1473,7 +1474,7 @@ public class Util {
     }
 
     public static String findPigJarName() {
-        final String suffix = System.getProperty("hadoopversion").equals("20") ? "1" : "2";
+        final String suffix = System.getProperty("hadoopversion", "2");
         File baseDir = new File(".");
         String[] jarNames = baseDir.list(new FilenameFilter() {
             @Override
